/*******************************************************************************
 * Copyright (c) {2007-2008} {INRIA and Engineering}.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    {Adrian Mos (INRIA) and Andrea Zoppello (Engineering)} - initial API and implementation
 *******************************************************************************/
package org.eclipse.stp.im.provider;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.edit.provider.ChangeNotifier;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IChangeNotifier;
import org.eclipse.emf.edit.provider.IDisposable;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;

import org.eclipse.stp.im.util.ImAdapterFactory;

/**
 * This is the factory that is used to provide the interfaces needed to support Viewers.
 * The adapters generated by this factory convert EMF adapter notifications into calls to {@link #fireNotifyChanged fireNotifyChanged}.
 * The adapters also support Eclipse property sheets.
 * Note that most of the adapters are shared among multiple instances.
 * <!-- begin-user-doc -->
 * <!-- end-user-doc -->
 * @generated
 */
public class ImItemProviderAdapterFactory extends ImAdapterFactory implements ComposeableAdapterFactory, IChangeNotifier, IDisposable {
	/**
	 * This keeps track of the root adapter factory that delegates to this adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ComposedAdapterFactory parentAdapterFactory;

	/**
	 * This is used to implement {@link org.eclipse.emf.edit.provider.IChangeNotifier}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IChangeNotifier changeNotifier = new ChangeNotifier();

	/**
	 * This keeps track of all the supported types checked by {@link #isFactoryForType isFactoryForType}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected Collection<Object> supportedTypes = new ArrayList<Object>();

	/**
	 * This constructs an instance.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ImItemProviderAdapterFactory() {
		supportedTypes.add(IEditingDomainItemProvider.class);
		supportedTypes.add(IStructuredItemContentProvider.class);
		supportedTypes.add(ITreeItemContentProvider.class);
		supportedTypes.add(IItemLabelProvider.class);
		supportedTypes.add(IItemPropertySource.class);
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Process} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessItemProvider processItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Process}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createProcessAdapter() {
		if (processItemProvider == null) {
			processItemProvider = new ProcessItemProvider(this);
		}

		return processItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Step} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StepItemProvider stepItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Step}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createStepAdapter() {
		if (stepItemProvider == null) {
			stepItemProvider = new StepItemProvider(this);
		}

		return stepItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Service} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ServiceItemProvider serviceItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Service}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createServiceAdapter() {
		if (serviceItemProvider == null) {
			serviceItemProvider = new ServiceItemProvider(this);
		}

		return serviceItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ServiceClassification} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ServiceClassificationItemProvider serviceClassificationItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ServiceClassification}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createServiceClassificationAdapter() {
		if (serviceClassificationItemProvider == null) {
			serviceClassificationItemProvider = new ServiceClassificationItemProvider(this);
		}

		return serviceClassificationItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Transition} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected TransitionItemProvider transitionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Transition}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createTransitionAdapter() {
		if (transitionItemProvider == null) {
			transitionItemProvider = new TransitionItemProvider(this);
		}

		return transitionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Owner} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected OwnerItemProvider ownerItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Owner}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createOwnerAdapter() {
		if (ownerItemProvider == null) {
			ownerItemProvider = new OwnerItemProvider(this);
		}

		return ownerItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.TransitionUnderCondition} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected TransitionUnderConditionItemProvider transitionUnderConditionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.TransitionUnderCondition}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createTransitionUnderConditionAdapter() {
		if (transitionUnderConditionItemProvider == null) {
			transitionUnderConditionItemProvider = new TransitionUnderConditionItemProvider(this);
		}

		return transitionUnderConditionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ObservableAttribute} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ObservableAttributeItemProvider observableAttributeItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ObservableAttribute}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createObservableAttributeAdapter() {
		if (observableAttributeItemProvider == null) {
			observableAttributeItemProvider = new ObservableAttributeItemProvider(this);
		}

		return observableAttributeItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Property} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PropertyItemProvider propertyItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Property}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createPropertyAdapter() {
		if (propertyItemProvider == null) {
			propertyItemProvider = new PropertyItemProvider(this);
		}

		return propertyItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ProcessCollection} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ProcessCollectionItemProvider processCollectionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ProcessCollection}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createProcessCollectionAdapter() {
		if (processCollectionItemProvider == null) {
			processCollectionItemProvider = new ProcessCollectionItemProvider(this);
		}

		return processCollectionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ServiceBinding} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ServiceBindingItemProvider serviceBindingItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ServiceBinding}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createServiceBindingAdapter() {
		if (serviceBindingItemProvider == null) {
			serviceBindingItemProvider = new ServiceBindingItemProvider(this);
		}

		return serviceBindingItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ControlService} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ControlServiceItemProvider controlServiceItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ControlService}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createControlServiceAdapter() {
		if (controlServiceItemProvider == null) {
			controlServiceItemProvider = new ControlServiceItemProvider(this);
		}

		return controlServiceItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.RouterControl} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected RouterControlItemProvider routerControlItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.RouterControl}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createRouterControlAdapter() {
		if (routerControlItemProvider == null) {
			routerControlItemProvider = new RouterControlItemProvider(this);
		}

		return routerControlItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.SplitControl} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected SplitControlItemProvider splitControlItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.SplitControl}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createSplitControlAdapter() {
		if (splitControlItemProvider == null) {
			splitControlItemProvider = new SplitControlItemProvider(this);
		}

		return splitControlItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.JoinControl} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected JoinControlItemProvider joinControlItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.JoinControl}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createJoinControlAdapter() {
		if (joinControlItemProvider == null) {
			joinControlItemProvider = new JoinControlItemProvider(this);
		}

		return joinControlItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.IterationControl} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected IterationControlItemProvider iterationControlItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.IterationControl}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createIterationControlAdapter() {
		if (iterationControlItemProvider == null) {
			iterationControlItemProvider = new IterationControlItemProvider(this);
		}

		return iterationControlItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.StpIntermediateModel} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StpIntermediateModelItemProvider stpIntermediateModelItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.StpIntermediateModel}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createStpIntermediateModelAdapter() {
		if (stpIntermediateModelItemProvider == null) {
			stpIntermediateModelItemProvider = new StpIntermediateModelItemProvider(this);
		}

		return stpIntermediateModelItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ServiceCollection} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ServiceCollectionItemProvider serviceCollectionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ServiceCollection}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createServiceCollectionAdapter() {
		if (serviceCollectionItemProvider == null) {
			serviceCollectionItemProvider = new ServiceCollectionItemProvider(this);
		}

		return serviceCollectionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.BasicProperty} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected BasicPropertyItemProvider basicPropertyItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.BasicProperty}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createBasicPropertyAdapter() {
		if (basicPropertyItemProvider == null) {
			basicPropertyItemProvider = new BasicPropertyItemProvider(this);
		}

		return basicPropertyItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.MapProperty} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected MapPropertyItemProvider mapPropertyItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.MapProperty}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createMapPropertyAdapter() {
		if (mapPropertyItemProvider == null) {
			mapPropertyItemProvider = new MapPropertyItemProvider(this);
		}

		return mapPropertyItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Contract} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ContractItemProvider contractItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Contract}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createContractAdapter() {
		if (contractItemProvider == null) {
			contractItemProvider = new ContractItemProvider(this);
		}

		return contractItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link java.util.Map.Entry} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected StringToPropertyMapEntryItemProvider stringToPropertyMapEntryItemProvider;

	/**
	 * This creates an adapter for a {@link java.util.Map.Entry}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createStringToPropertyMapEntryAdapter() {
		if (stringToPropertyMapEntryItemProvider == null) {
			stringToPropertyMapEntryItemProvider = new StringToPropertyMapEntryItemProvider(this);
		}

		return stringToPropertyMapEntryItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Endpoint} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected EndpointItemProvider endpointItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Endpoint}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createEndpointAdapter() {
		if (endpointItemProvider == null) {
			endpointItemProvider = new EndpointItemProvider(this);
		}

		return endpointItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.PropertyCondition} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected PropertyConditionItemProvider propertyConditionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.PropertyCondition}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createPropertyConditionAdapter() {
		if (propertyConditionItemProvider == null) {
			propertyConditionItemProvider = new PropertyConditionItemProvider(this);
		}

		return propertyConditionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ExpressionCondition} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExpressionConditionItemProvider expressionConditionItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ExpressionCondition}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createExpressionConditionAdapter() {
		if (expressionConditionItemProvider == null) {
			expressionConditionItemProvider = new ExpressionConditionItemProvider(this);
		}

		return expressionConditionItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.ExtractDataRule} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected ExtractDataRuleItemProvider extractDataRuleItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.ExtractDataRule}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createExtractDataRuleAdapter() {
		if (extractDataRuleItemProvider == null) {
			extractDataRuleItemProvider = new ExtractDataRuleItemProvider(this);
		}

		return extractDataRuleItemProvider;
	}

	/**
	 * This keeps track of the one adapter used for all {@link org.eclipse.stp.im.Variable} instances.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected VariableItemProvider variableItemProvider;

	/**
	 * This creates an adapter for a {@link org.eclipse.stp.im.Variable}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter createVariableAdapter() {
		if (variableItemProvider == null) {
			variableItemProvider = new VariableItemProvider(this);
		}

		return variableItemProvider;
	}

	/**
	 * This returns the root adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public ComposeableAdapterFactory getRootAdapterFactory() {
		return parentAdapterFactory == null ? this : parentAdapterFactory.getRootAdapterFactory();
	}

	/**
	 * This sets the composed adapter factory that contains this factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void setParentAdapterFactory(ComposedAdapterFactory parentAdapterFactory) {
		this.parentAdapterFactory = parentAdapterFactory;
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object type) {
		return supportedTypes.contains(type) || super.isFactoryForType(type);
	}

	/**
	 * This implementation substitutes the factory itself as the key for the adapter.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Adapter adapt(Notifier notifier, Object type) {
		return super.adapt(notifier, this);
	}

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	@Override
	public Object adapt(Object object, Object type) {
		if (isFactoryForType(type)) {
			Object adapter = super.adapt(object, type);
			if (!(type instanceof Class) || (((Class<?>)type).isInstance(adapter))) {
				return adapter;
			}
		}

		return null;
	}

	/**
	 * This adds a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void addListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.addListener(notifyChangedListener);
	}

	/**
	 * This removes a listener.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void removeListener(INotifyChangedListener notifyChangedListener) {
		changeNotifier.removeListener(notifyChangedListener);
	}

	/**
	 * This delegates to {@link #changeNotifier} and to {@link #parentAdapterFactory}.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void fireNotifyChanged(Notification notification) {
		changeNotifier.fireNotifyChanged(notification);

		if (parentAdapterFactory != null) {
			parentAdapterFactory.fireNotifyChanged(notification);
		}
	}

	/**
	 * This disposes all of the item providers created by this factory. 
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public void dispose() {
		if (processItemProvider != null) processItemProvider.dispose();
		if (stepItemProvider != null) stepItemProvider.dispose();
		if (serviceItemProvider != null) serviceItemProvider.dispose();
		if (serviceClassificationItemProvider != null) serviceClassificationItemProvider.dispose();
		if (transitionItemProvider != null) transitionItemProvider.dispose();
		if (ownerItemProvider != null) ownerItemProvider.dispose();
		if (transitionUnderConditionItemProvider != null) transitionUnderConditionItemProvider.dispose();
		if (observableAttributeItemProvider != null) observableAttributeItemProvider.dispose();
		if (propertyItemProvider != null) propertyItemProvider.dispose();
		if (processCollectionItemProvider != null) processCollectionItemProvider.dispose();
		if (serviceBindingItemProvider != null) serviceBindingItemProvider.dispose();
		if (controlServiceItemProvider != null) controlServiceItemProvider.dispose();
		if (routerControlItemProvider != null) routerControlItemProvider.dispose();
		if (splitControlItemProvider != null) splitControlItemProvider.dispose();
		if (joinControlItemProvider != null) joinControlItemProvider.dispose();
		if (iterationControlItemProvider != null) iterationControlItemProvider.dispose();
		if (stpIntermediateModelItemProvider != null) stpIntermediateModelItemProvider.dispose();
		if (serviceCollectionItemProvider != null) serviceCollectionItemProvider.dispose();
		if (basicPropertyItemProvider != null) basicPropertyItemProvider.dispose();
		if (mapPropertyItemProvider != null) mapPropertyItemProvider.dispose();
		if (contractItemProvider != null) contractItemProvider.dispose();
		if (stringToPropertyMapEntryItemProvider != null) stringToPropertyMapEntryItemProvider.dispose();
		if (endpointItemProvider != null) endpointItemProvider.dispose();
		if (propertyConditionItemProvider != null) propertyConditionItemProvider.dispose();
		if (expressionConditionItemProvider != null) expressionConditionItemProvider.dispose();
		if (extractDataRuleItemProvider != null) extractDataRuleItemProvider.dispose();
		if (variableItemProvider != null) variableItemProvider.dispose();
	}

}
