/*******************************************************************************
 * Copyright (c) {2007, 2008} Engineering Ingegneria Informatica S.p.A.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Andrea Zoppello (Engineering) - initial API and implementation
 *******************************************************************************/
package org.eclipse.stp.im.tool.in.bpmneditor.section;


import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.Graph;
import org.eclipse.stp.bpmn.Pool;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.eclipse.stp.im.tool.in.bpmneditor.BpmnEditorExtensionActivator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

public class TechnologySection  extends AbstractPropertySection implements SelectionListener {
	/**
     * Creates the UI of the section.
     */
	 protected CLabel poolLabel = null;
	 protected CLabel runtimeLabel = null;
	 protected CCombo  technologyText = null;
	 protected Button btnSetTechnology = null;
	 protected Pool pool = null;
	 protected String[] data = null;
	 protected TabbedPropertySheetPage page = null;

	 @Override
     public void createControls(Composite parent, 
             TabbedPropertySheetPage aTabbedPropertySheetPage) {
             
		 	 super.createControls(parent, aTabbedPropertySheetPage);
             
		 	 FontData fTextFontData = new FontData( );

		 	 fTextFontData.setStyle(SWT.BOLD);
		 	 fTextFontData.setName("Tahoma");
		 	 fTextFontData.setHeight(8);
		 	 Font boldText = new Font(PlatformUI.getWorkbench().getDisplay(),fTextFontData);

		 	
             org.eclipse.swt.layout.GridLayout layout = new GridLayout(2, false);
             parent.setLayout(layout);
             
             page = aTabbedPropertySheetPage;
             GridData gd = new GridData(SWT.FILL);
             gd.minimumWidth = 500;
             gd.widthHint = 500;
             
             
             // Row Containing "Pool : <POOL_NAME> "
             CLabel label2 = getWidgetFactory().createCLabel(parent, "Pool");
             label2.setFont(boldText);
             
             poolLabel = getWidgetFactory().createCLabel(parent, "");
             poolLabel.setLayoutData(gd);
             
             // Row Containing "Runtime  : <POOL_NAME> "
             CLabel label3 = getWidgetFactory().createCLabel(parent, "Runtime");
             label3.setFont(boldText);
             
             runtimeLabel = getWidgetFactory().createCLabel(parent, "Not Defined");
             runtimeLabel.setLayoutData(gd);
             
          
             this.technologyText = getWidgetFactory().createCCombo(parent, SWT.DROP_DOWN | SWT.SINGLE |SWT.V_SCROLL | SWT.H_SCROLL | SWT.READ_ONLY );
             //this.technologyText.addSelectionListener(this);
             this.data = ImRuntimeActivator.getRuntimeNames();
                
             this.technologyText.setItems(this.data);
             this.technologyText.select(0);
             
             GridData gd4 = new GridData(SWT.FILL);
             gd4.minimumWidth = 100;
             gd4.widthHint = 100;
             this.btnSetTechnology = getWidgetFactory().createButton(parent, " Bind to Runtime ", SWT.PUSH);
             this.btnSetTechnology.setLayoutData(gd4);
             this.btnSetTechnology.addSelectionListener(this);
     }
	 
	 
     
     
     /**
      * Manages the input.
      */
      @Override
      public void setInput(IWorkbenchPart part, ISelection selection) {
      
              super.setInput(part, selection);
              if(selection instanceof IStructuredSelection) {
                      Object unknownInput = 
                             ((IStructuredSelection) selection).getFirstElement();
                      if (unknownInput instanceof IGraphicalEditPart && 
                              (((IGraphicalEditPart) unknownInput).
                                      resolveSemanticElement() != null)) {
                               unknownInput = ((IGraphicalEditPart) unknownInput).resolveSemanticElement();
                      }
                      if (unknownInput instanceof Pool) {
                           pool = (Pool)unknownInput;	 
                           this.technologyText.setEnabled(true);
                           this.btnSetTechnology.setEnabled(true);
                     	   EAnnotation technologyAnnotation = pool.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION);
                           if ( technologyAnnotation != null){
                          		String technologyName = technologyAnnotation.getDetails().get(ImConstants.TECHNOLOGY_NAME);
                          		runtimeLabel.setText(technologyName);
                          		this.technologyText.select(findSelectionIndex( technologyName));
                            }
                          	return;
                      }else if (unknownInput instanceof Activity) {
                    	  Activity activity  = (Activity)unknownInput;
                    	  Graph graph = activity.getGraph();
                    	  if ((graph instanceof Pool) && (graph.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION) != null )) {
                    		  this.technologyText.setEnabled(false);
                    		  this.btnSetTechnology.setEnabled(false);
                              this.technologyText.select(findSelectionIndex( graph.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION).getDetails().get(ImConstants.TECHNOLOGY_NAME)));
                              return;
                    	  }
                      }else{   
                    	  
                      }
              }     

              
              this.technologyText.setEnabled(false);
              this.btnSetTechnology.setEnabled(false);

      }
      
      /**
  	 * Creates an EAnnotation with a source parameter and
  	 * a details map.
  	 * @param source
  	 * @param details
  	 * @return
  	 */
  	protected EAnnotation createAnnotation(String source, Map<String, String> details) {
  		EAnnotation ea = EcoreFactory.eINSTANCE.createEAnnotation();
  		ea.setSource(source);
  		ea.getDetails().putAll(details);
  		return ea;
  	}
  	
  	private abstract class CreateOrUpdateTechnologyEAnnotationCommand extends AbstractTransactionalCommand {
        
        public CreateOrUpdateTechnologyEAnnotationCommand(Pool pool, String label) {
                 super((TransactionalEditingDomain) AdapterFactoryEditingDomain.getEditingDomainFor(pool), label, getWorkspaceFiles(pool));
        }
  	}

  	




	public void widgetDefaultSelected(SelectionEvent ev) {
		widgetSelected(ev);
	}




	public void widgetSelected(SelectionEvent ev) {
		
		
		 if (ev.getSource() instanceof Button) {

			 	int selectionIndex = this.technologyText.getSelectionIndex();
				
				final String technologyName = data[selectionIndex];
				final String technologyId = ImRuntimeActivator.getIdForRuntimeName(technologyName);
				String previousTechnologyId = null;
				String previousTechnologyName = null;
				if (pool.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION) != null){
					previousTechnologyId = pool.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION).getDetails().get(ImConstants.IM_POOL_RUNTIME_ID);
					previousTechnologyName = pool.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION).getDetails().get(ImConstants.TECHNOLOGY_NAME);
				}
				
				boolean safeUpdate = false;
				
			
				if (technologyId.equalsIgnoreCase(previousTechnologyId)){
					safeUpdate = true;
				}else if ( previousTechnologyId == null || previousTechnologyId.equalsIgnoreCase("AbstractModel")){
					//
					// Changing from technology Abstract to specific technology
					//
					safeUpdate = true;
				}
				
				System.out.println(" Technology Change ["+previousTechnologyId+"] --> ["+technologyId+"] Update Safe ["+safeUpdate+"]");
				
				if (safeUpdate){
					CreateOrUpdateTechnologyEAnnotationCommand command = new CreateOrUpdateTechnologyEAnnotationCommand(pool, "Modifying Technology") {
		                     
						@Override
						protected CommandResult doExecuteWithResult(IProgressMonitor arg0, IAdaptable arg1) throws ExecutionException {
		            	 	EAnnotation poolTechnologyAnnotation = pool.getEAnnotation(ImConstants.TECHNOLOGY_ANNOTATION);
              	   
		            	 	if (poolTechnologyAnnotation == null){
		            	 		Map<String, String> details = new HashMap<String, String>();
		            	 		details.put(ImConstants.IM_POOL_RUNTIME_ID, ImRuntimeActivator.getIdForRuntimeName(technologyName));
		            	 		details.put(ImConstants.TECHNOLOGY_NAME,technologyName);
              		   
		            	 		poolTechnologyAnnotation = createAnnotation(ImConstants.TECHNOLOGY_ANNOTATION, details);
		            	 		SetCommand.create(getEditingDomain(), poolTechnologyAnnotation, 
		            	 				EcorePackage.eINSTANCE.getEAnnotation_EModelElement(),pool).execute();
		            	 	}else{
		            	 		poolTechnologyAnnotation.getDetails().put(ImConstants.TECHNOLOGY_NAME,technologyName);   
		            	 		poolTechnologyAnnotation.getDetails().put(ImConstants.IM_POOL_RUNTIME_ID,ImRuntimeActivator.getIdForRuntimeName(technologyName));
		            	 	}
		            	 	return CommandResult.newOKCommandResult();
						}
					};
		            try {
		                    command.execute(new NullProgressMonitor(), null);
		            } catch (ExecutionException exception) {
		            	BpmnEditorExtensionActivator.getDefault().getLog().log(new Status(IStatus.ERROR, BpmnEditorExtensionActivator.PLUGIN_ID, IStatus.ERROR, exception.getMessage(), exception));
		            }
		            this.runtimeLabel.setText(technologyName);
				}else{
					MessageDialog.openError(page.getSite().getShell(), " Error "," Technology Change ["+previousTechnologyId+"] --> ["+technologyId+"] Cannot be done");
					this.technologyText.select(findSelectionIndex(previousTechnologyName));
				}
			
		}

	}
	
	public int findSelectionIndex(String str){
		for (int i = 0; i < data.length; i++){
			if (data[i].equalsIgnoreCase(str))
				return i;
		}
		return 0;
	}
}
